unit sgCommonFunctions;

{$INCLUDE SGDXF.INC}

interface

uses
  sgConsts, sgFunction, Classes, Graphics, Types, CADImage, SysUtils, Windows,
  ShFolder, ShlObj, Dialogs, Controls, Forms, ActiveX, Registry, ComObj, SHX,
  Variants, StdCtrls, Printers, Consts, ShellAPI, Math, TypInfo
{$IFDEF SGMULLANG}, sgMultiLang{$ENDIF}
{$IFDEF SGDEL_XE2}
  , System.UITypes
{$ENDIF}
  ;

const
  cnstVersionBeta = ' beta';
  cnstVersionX64bit = ' x64';
  iControlOffset = 8;
  cnstControlsOffset = iControlOffset;
  sSoftGold = cnstCompany;
  sSGInfo = 'Soft Gold Ltd  2003-%s All rights reserved.';
  sSGInfoEx: string = sSGInfo + ' Address: 300002, Tula-2, p.o.box 248, Russia';
  SUnlimited: string = 'unlimited';
  sUnregistered_User = 'Unregistered user';
  sRegistered_User: string = 'Registered user';
  sRegistered_User_Default = 'Registered user';
  sNoSHXPaths: string = 'There are no AutoCAD SHX fonts paths';
  sNoPrintersInSystem: string = 'No printers in system.';
  sPrintersError: string = 'There are no connected printers.';
  BadValue: string = 'Value incorrect';
  sColor: string = 'Color';
  cnstClose = 'Close';
  sClose: string = cnstClose;
  sLineweight: string = 'Lineweight';
  sInvalidNameError = 'The name can use up to 255 characters, including letters, numbers, '#13#10'hyphens, underscores, spaces, and dollar sign ($), but not %s.'#13#10'Please try again.';
  sPleaseEnterUniqueName =  'You have entered a style name that already exists in then drawing. '+
    'Please enter a unique name for this style';
  sTextStyles = 'Text Style';
  sStyleName = '&Style name:';
  sLeft = 'Left';
  sTop = 'Top';
  sWidth = 'Width';
  sHeight = 'Height';
  cnstInchString = 'inch';
  sInch: string = cnstInchString;
  cnstMMString = 'mm';
  sMM: string = cnstMMString;
  sM: string = 'm';
  sLWDefault: string = 'Default';
  cnstAppNameMacro = '[AppName]';
  cnstFileNotFound = 'File not found:';
  cnstInputBoxFormName = 'InputBoxForm';
  cnstInputBoxEditName = 'InputBoxEdit';
  cnstInputBoxLabelName = 'InputBoxLabel';
  cnstInputBoxBtnOKName = 'InputBoxBtnOK';
  cnstInputBoxBtnCancelName = 'InputBoxBtnCancel';

{$IFDEF SG_EVACUATION}
  cnstVersionLabelColor = $508D6D;
  cnstLabelHorIdentPercent = 0.808;
  cnstLabelVerIdentPercent = 0.167;
{$ELSE}
  {$IFDEF SG_INVENTORY}
  cnstVersionLabelColor = $4449AD;
  cnstLabelHorIdentPercent = 0.552;
  cnstLabelVerIdentPercent = 0.167;
  {$ELSE}
  cnstVersionLabelColor = $FFA30A;
    {$IFDEF SG_CADVIEWX_LITE} // CADViewX Lite
  cnstLabelHorIdentPercent = 0.79;
  cnstLabelVerIdentPercent = 0.167;
    {$ELSE}
      {$IFDEF SG_CADVIEWX}    // CADViewX Pro
  cnstLabelHorIdentPercent = 0.77;
  cnstLabelVerIdentPercent = 0.167;
      {$ELSE}
  cnstLabelHorIdentPercent = 0.552;
  cnstLabelVerIdentPercent = 0.167;
      {$ENDIF}
    {$ENDIF}

  {$ENDIF}
{$ENDIF}

//Alings an instance of the TConstrol class on a TWinControl Parent as alClient
//with offsets
procedure AlignAsClient(AParent, AControl: TWinControl; ALeft, ATop: Integer);
//Alings TButtons on the Parent. Parent must be a TWinControl descendant class
//All TButtons must be ranged by Tag property, First is 0 Second is 1 Third is 2 etc.
procedure AlignButtonsOnPanel(AParent: TWinControl; ATop: Integer);
function CreateFolder(AName: string): Boolean;
function CutFileFromArchiveName(var AArchiveFile: string): string;
function CheckFloatingPointAccuracy(AEdit: TEdit): Boolean; overload;
function CheckFloatingPointAccuracy(AString: string): Boolean; overload;
//Creates an instance of the TClass as a Component class
function CreateComponent(AClass: TClass; AOwner: TComponent;
  AParent: TWinControl; const APos: TRect; ATabOrder: Integer;
  AOnClickEvent: TNotifyEvent; AComtForAignment: TControl;
  AVisible: Boolean = True): TComponent;
function CreateInpBox(ACaption, ALabel: string; const AText: string;
  const AOnCreate: TNotifyEvent = nil): TForm;
function CreateMetafileFromCadImage(const AImg: TsgCADImage;
  const AIsEnhanced: Boolean; const APicWidth, APicHeight, APicInch: Integer;
  const ADrawProc: TsgObjProcDraw; const AUseLogicMetrix: Boolean): TMetafile;
function CreateNewCrossCursor(const ASizeX, ASizeY, AApertureSize: Integer;
  const AMaskColor: TColor = 0; const AXorColor: TColor = $FFFFFF): HICON;
function DrawLineweight(Sender: TObject; Rect: TRect; ACanvas: TCanvas;
  AWeight: Double; State: TOwnerDrawState; AText: string): Boolean;
function ExpandEnvironmentStr(const AStr: string): string;
procedure FillItemsByLineweights(const AItems: TStrings);
procedure FloatCheckKeyPress(Sender: TObject; var Key: Char; IsUnsigned: Boolean);
function FindCompByClass(AOwner: TComponent; AClass: TComponentClass;
  var I: Integer): Boolean;
function FloatToStrWeight(AValue: Double): string;
function GetApplicationVersion: string;
function GetApplicationMainVersion: string;
function GetNetPath(var APath: string): Boolean;
function GetProductSettingCommonPath(AProduct: string): string;
function GetProductSettingsSavePath(AProduct: string): string;
function GetUniqueComponentName(AOwner: TComponent; const APreffix: string): string;
function GetWindowNormalPosition(AForm: TCustomForm): TRect;
//Returnts maximum width of the TCheckbox
function GetMaxCheckBoxWidth(const ACheckbox: TCheckbox): Integer;
function GetFileVersionStr(AFileName: string): string;
function GetFileResInfo(const AFileName, AItem: string): string;
function GetMetafileSizeFromCadImage(const AImg: TsgCADImage): TPoint;
function IsExistPrinterInSystem: Boolean;
function InpBox(ACaption, ALabel: string; var AText: string;
  const AOnCreate: TNotifyEvent = nil): Boolean;
function IsEqualMethod(AMethod1, AMethod2: TMethod): Boolean;
function IsEqualEvent(AEvent1, AEvent2: TNotifyEvent): Boolean;
function IsSavePathValid(const APathName: string): Boolean;
function LoadCloseIcon: Graphics.TBitmap;
function sgCreateDir(S: string): Boolean;
procedure sgTextRect(ACanvasHandle: THandle; var ARect: TRect;
  var AText: string; ATextFormat: TTextFormat = []);
procedure IntegerCheckKeyPress(Sender: TObject; var Key: Char; IsUnsigned: Boolean);
function StrToWeight(S: string): Double;
function ShowFormatMessage(const AMessage: string; const Args: array of const;
  DlgType: TMsgDlgType; Buttons: TMsgDlgButtons): Integer;
function sgInformationMessage(const AMessage: string): Integer;
function sgErrorMessage(const AMessage: string): Integer;
function ShellOpen(const AFileName: string; const AHandle: Cardinal = 0): Cardinal;
function sgIsMethodEqual(AM1, AM2: TWndMethod): Boolean;
function sgNotWrapMessageDialog(const Msg: string; DlgType: TMsgDlgType;
  Buttons: TMsgDlgButtons): Integer;
procedure sgScanDirectory(const APath, AFileMask: string; AFiles: TStrings;
  ASearchSubfolders: Boolean = True);
procedure ResetSHXFontsHash;
function WebFileExt(const AFilelName: string): string;

implementation

type
  TsgLabelAccess = class(TLabel);
  TsgCADImageAccess = class(TsgCADImage);

function GetApplicationVersion: string;
begin
  if IsLibrary then
    Result := GetFileVersionStr(GetModuleName(HInstance))
  else
    Result := GetFileVersionStr(ParamStr(0));
{$IFDEF SG_BETA}
  Result := Result + cnstVersionBeta;
{$ENDIF}
end;

function GetApplicationMainVersion: string;
var
  I{$IFDEF SG_BETA}, J{$ENDIF}: Integer;
begin
  Result := GetApplicationVersion;
  if Length(Result) > 0 then
  begin
    I := sgFunction.StringScan('.', Result, 1);
    if I > 0 then
    begin
{$IFDEF SG_BETA}
      J := sgFunction.StringScan('.', Result, I + 1);
      if J > 0 then
        I := J;
{$ENDIF}
      SetLength(Result, I - 1);
    end;
  end;
{$IFDEF SG_BETA}
  Result := Result + cnstVersionBeta;
{$ENDIF}
end;

function GetNetPath(var APath: string): Boolean;
var
  ShellObject: IUnknown;
  Link: IShellLink;
  LinkFile: IPersistFile;
  Arg: array[0..MAX_PATH] of Char;
  LinkPath: PWideChar;
  Data: TWin32FindData;
  vPath: string;

  function GetNetHoodPath: string;
  var
    Reg: TRegistry;
  begin
    Result := '';
    Reg := TRegistry.Create;
    try
      Reg.RootKey := HKEY_CURRENT_USER;
      if Reg.OpenKeyReadOnly('\Software\Microsoft\Windows\CurrentVersion\'+
                'Explorer\Shell Folders'
                ) then
      begin
        Result := Reg.ReadString('NetHood');
        Reg.CloseKey;
      end;
    finally
      Reg.Free;
    end;
  end;

begin
  Result := False;
  vPath := APath;
  if vPath  = '' then Exit;
  if vPath [Length(vPath)] = '\' then Delete(vPath, Length(vPath), 1);
  if AnsiPos(AnsiUpperCase(GetNetHoodPath), AnsiUpperCase(vPath)) > 0 then
  begin
    if FileExists(vPath  + '\target.lnk') then
    begin
      try
        CoInitialize(nil);
        ShellObject := CreateComObject(CLSID_ShellLink);
        LinkFile := ShellObject as IPersistFile;
        LinkPath := PWChar(WideString(vPath + '\target.lnk'));
        if LinkFile.Load(LinkPath, STGM_READ) = S_OK then
        begin
          Link := ShellObject as IShellLink;
          FillChar(Arg, SizeOf(Arg), #0);
          if Link.GetPath(Arg, MAX_PATH, Data, SLGP_SHORTPATH) = S_OK then
            vPath := string(Arg);
        end;
        Result := True;
        APath := vPath;
      finally
        ShellObject := Unassigned;
        CoUninitialize;
      end;
    end
    else
      Exit;
  end
  else
    Result := True;
end;

function ExpandEnvironmentStr(const AStr: string): string;
var
  vBuffer: array[0 .. MAX_PATH - 1] of Char;
begin
  FillChar(vBuffer, Length(vBuffer) * SizeOf(Char), 0);
  ExpandEnvironmentStrings(PChar(AStr), vBuffer, Length(vBuffer));
  SetString(Result, vBuffer, StrLen(vBuffer));
end;

function IsExistPrinterInSystem: Boolean;
begin
  Result := True;
  try
    Printer.Refresh;
    if Printer.Printers.Count > 0 then
      Exit;
    Result := False;
    MessageBox(Application.Handle, PChar(sNoPrintersInSystem), 'Error', 0);
  except
    on E: Exception do
    begin
      MessageBox(Application.Handle, PChar(sPrintersError), 'Error', 0);
      Result := False;
    end;
  end;
end;

procedure AlignAsClient(AParent, AControl: TWinControl; ALeft, ATop: Integer);
begin
  if AControl = nil then Exit;
  AControl.Left := ALeft;
  AControl.Top := ATop;
  AControl.Width := AParent.ClientWidth - ALeft - cnstControlsOffset;
  AControl.Height := AParent.ClientHeight - ATop - cnstControlsOffset;
end;

procedure AlignButtonsOnPanel(AParent: TWinControl; ATop: Integer);
var
  I: Integer;
  vButtons: array of TButton;
  vButton, vPrevButton: TButton;
begin
  SetLength(vButtons, AParent.ControlCount);
  FillChar(vButtons[0], SizeOf(TButton) * Length(vButtons), Integer(nil));
  for I := 0 to AParent.ControlCount - 1 do
  begin
    if AParent.Controls[I] is TButton then
      vButtons[AParent.Controls[I].Tag] := TButton(AParent.Controls[I]);
  end;
  vPrevButton := nil;
  for I := Low(vButtons) to High(vButtons) do
  begin
    vButton := vButtons[I];
    if vButton = nil then Continue;
    vButton.Top := ATop;
    if I = Low(vButtons) then
      vButton.Left := AParent.ClientWidth - vButton.Width - cnstControlsOffset
    else
    begin
      if vPrevButton = nil then Continue;
      vButton.Left := vPrevButton.Left - vButton.Width - cnstControlsOffset;
      vButton.Height := vPrevButton.Height;
    end;
    vPrevButton := vButton;
  end;

  for I := Low(vButtons) to High(vButtons) do
    if vButtons[I] <> nil then
    begin
      AParent.ClientHeight := vButtons[I].Top + vButtons[I].Height +
        cnstControlsOffset;
      Break;
    end;
end;

function CreateFolder(AName: string): Boolean;
var
  Ch: PChar;
begin
  Ch := PChar(AName);
  Result := CreateDirectory(Ch, nil);
end;

function CutFileFromArchiveName(var AArchiveFile: string): string;
var
  vPos: Integer;
begin
  Result := '';
  vPos := AnsiPos('/', AArchiveFile);
  if vPos > 0 then
  begin
    Result := Copy(AArchiveFile, vPos + 1, Length(AArchiveFile) - vPos);
    Delete(AArchiveFile, vPos, Length(AArchiveFile) - vPos + 1);
  end;
end;

function sgCreateDir(S: string): Boolean;
begin
  Result := True;
  if not DirectoryExists(S) then
    if not CreateDirectory(PChar(S), nil) then
      Result := False;
end;

procedure sgTextRect(ACanvasHandle: THandle; var ARect: TRect;
  var AText: string; ATextFormat: TTextFormat = []);
const
  cTextFormats: array[TTextFormats] of Integer =
  (DT_BOTTOM, DT_CALCRECT, DT_CENTER, DT_EDITCONTROL, DT_END_ELLIPSIS,
   DT_PATH_ELLIPSIS, DT_EXPANDTABS, DT_EXTERNALLEADING, DT_LEFT,
   DT_MODIFYSTRING, DT_NOCLIP, DT_NOPREFIX, DT_RIGHT, DT_RTLREADING,
   DT_SINGLELINE, DT_TOP, DT_VCENTER, DT_WORDBREAK
{$IFDEF SGDEL_XE2},
   DT_HIDEPREFIX, DT_NOFULLWIDTHCHARBREAK, DT_PREFIXONLY, DT_TABSTOP,
   DT_WORD_ELLIPSIS, 0
{$ENDIF}
  );
var
  vFormat: Integer;
  vF: TTextFormats;
  vRealLength: Integer;
begin
  vFormat := 0;
  for vF := Low(TTextFormats) to High(TTextFormats) do
    if vF in ATextFormat then
      vFormat := vFormat or cTextFormats[vF];
  vRealLength := Length(AText);
  if tfModifyString in ATextFormat then
  begin
    AText := AText + #0#0#0#0;
  end;
  DrawTextEx(ACanvasHandle, PChar(AText), vRealLength, ARect, vFormat, nil);
  if tfModifyString in ATextFormat then
    SetLength(AText, StrLen(PChar(AText)));
end;


function GetPathByIDL(AProduct: string; const IDL: Integer): string;
var
  Path: PChar;
  S: string;
begin
  Result := ExtractFilePath(Application.ExeName);
  if Win32Platform = VER_PLATFORM_WIN32_NT then
  begin
    Path := StrAlloc(MAX_PATH);
    try
      if SHGetSpecialFolderPath(Application.Handle, Path, IDL, False) = True then
      begin
        S := Path + string('\' + sSoftGold + '\');
        if not sgCreateDir(S) then
          Exit;
        S := S + AProduct + '\';
        if not sgCreateDir(S) then
          Exit;
      end
      else
        Exit;
      Result := S;
    finally
      StrDispose(Path);
    end
  end
end;

function GetProductSettingCommonPath(AProduct: string): string;
begin
  Result := GetPathByIDL(AProduct, CSIDL_COMMON_APPDATA);
end;

function GetProductSettingsSavePath(AProduct: string): string;
begin
  Result := GetPathByIDL(AProduct, CSIDL_APPDATA);
end;

function GetUniqueComponentName(AOwner: TComponent; const APreffix: string): string;
var
  I: Integer;
begin
  Result := APreffix;
  if Assigned(AOwner) and not Assigned(AOwner.FindComponent(Result)) then
  begin
    I := 0;
    repeat
      Inc(I);
      Result := APreffix + IntToStr(I);
    until not Assigned(AOwner.FindComponent(Result));
  end;
end;

function GetWindowNormalPosition(AForm: TCustomForm): TRect;
var
  vpPlacement: PWindowPlacement;
begin
  New(vpPlacement);
  try
    vpPlacement.length := SizeOf(TWindowPlacement);
    GetWindowPlacement(AForm.Handle, vpPlacement);
    Result := vpPlacement^.rcNormalPosition;
  finally
    Dispose(vpPlacement);
  end;
end;

function GetMaxCheckBoxWidth(const ACheckbox: TCheckbox): Integer;
var
  vCanvas: TControlCanvas;
begin
  vCanvas := TControlCanvas.Create;
  try
    vCanvas.Control := ACheckbox;
    Result := GetSystemMetrics(SM_CXMENUCHECK) + 4 + vCanvas.TextWidth(ACheckbox.Caption);
  finally
    vCanvas.Free;
  end;
end;

function CheckFloatingPointAccuracy(AString: string): Boolean;
begin
  Delete(AString, Pos(GetDecimalSeparator, AString), 1);
  if Pos(GetDecimalSeparator, AString) <> 0 then
    Result := False
  else
    Result := True;
end;

type
  TControlAccess = class(TControl);

function CreateComponent(AClass: TClass; AOwner: TComponent;
  AParent: TWinControl; const APos: TRect; ATabOrder: Integer;
  AOnClickEvent: TNotifyEvent; AComtForAignment: TControl;
  AVisible: Boolean = True): TComponent;
begin
  Result := TComponentClass(AClass).Create(AOwner);
  if Result is TControl then
  begin
    TControl(Result).Parent := AParent;
    TControl(Result).Left := APos.Left;
    TControl(Result).Top := APos.Top;
    if APos.Right > 0 then
      TControl(Result).Width := APos.Right;
    if APos.Bottom > 0 then
      TControl(Result).Height := APos.Bottom;
    TControlAccess(Result).OnClick := AOnClickEvent;

    if Result is TWinControl then
      TWinControl(Result).TabOrder := ATabOrder;
    if AComtForAignment <> nil then
    begin
      TControl(Result).Left := AComtForAignment.Left + AComtForAignment.Width +
        cnstControlsOffset;
      TControl(Result).Top := AComtForAignment.Top;
    end;
{$IFDEF SGMULLANG}    if ComponentHasProperty(Result, 'Visible') then
      SetPropValue(Result, 'Visible', AVisible);
{$ENDIF}
  end;
end;

function CheckFloatingPointAccuracy(AEdit: TEdit): Boolean;
var
  S: string;

  procedure DoNotExit;
  begin
    AEdit.SetFocus;
    Result := False;
    Exit;
  end;

begin
  Result := True;
  S := AEdit.Text;
  if S = '' then
  begin
    AEdit.Text := '0';
    Result := False;
    Exit;
  end;
  if not CheckFloatingPointAccuracy(S) then
    DoNotExit;
end;

function FloatToStrWeight(AValue: Double): string;
var
  DS: Char;
begin
  DS := SetDecimalSeparator('.');
  try
    if AValue < 0 then
      Result := sLWDefault
    else Result := Format('%.2f ' + sMM, [AValue]);
  finally
    SetDecimalSeparator(DS);
  end;
end;

// convert, for example '0.25 mm', string to double 0.25
function StrToWeight(S: string): Double;
var
  vErr: Integer;
  DS: Char;
begin
  DS := GetDecimalSeparator;
  try
    vErr := Pos(DS, S);
    if vErr > 0 then
      S[vErr] := '.';
    SetDecimalSeparator('.');
    Val(S, Result, vErr); // Val() not work correctly, if DecimalSeparator <> '.'
    if vErr <> 0 then
    begin
      S := Copy(S, 1, vErr - 1);
      Val(S, Result, vErr);
    end;
    if vErr <> 0 then
      Result := fLineWeightDefault;
  finally
    SetDecimalSeparator(DS);
  end;
end;

procedure FillItemsByLineweights(const AItems: TStrings);
var
  I: Integer;
begin
  AItems.BeginUpdate;
  try
    AItems.Clear;
    AItems.AddObject(sByLayer, Pointer(Round(fLineWeightByLayer)));
    AItems.AddObject(sByBlock, Pointer(Round(fLineWeightByBlock)));
    AItems.AddObject(sLWDefault, Pointer(Round(fLineWeightDefault)));
    for I := Low(sgDXFLineWeights) to High(sgDXFLineWeights) do
      AItems.Add(FloatToStrWeight(sgDXFLineWeights[I] / 100.0));
  finally
    AItems.EndUpdate;
  end;
end;

function DrawLineweight(Sender: TObject; Rect: TRect; ACanvas: TCanvas;
  AWeight: Double; State: TOwnerDrawState; AText: string): Boolean;
var
  vR: TRect;
  vPenWidth: Integer;
  vWeight: Double;
  S: string;
  vLineWidth: Integer;
  vBrush: TBrush;
  vPen: TPen;
begin
  ACanvas.FillRect(Rect);
  vPen := TPen.Create;
  try
    if Sender <> nil then
      ACanvas.Font := TControlAccess(Sender).Font;
    vPen.Assign(ACanvas.Pen);
    vR := Rect;
    vWeight := AWeight;
    if vWeight = 0 then
      vWeight := sgDXFLineWeights[1] / 100.0
    else
      if vWeight < 0 then
        vWeight := sgDXFLineWeights[7] / 100.0; //0.25
    {
    if AWeight < 0 then
      S := sLWDefault
    else S := FloatToStrWeight(AWeight);
    }
    S := AText;
    UniqueString(S);
    vPenWidth := vR.Bottom - vR.Top;
    vLineWidth := vR.Right - vR.Left - ACanvas.TextWidth(S) - 4;
    vLineWidth := (vLineWidth div 8) * 8;
    if vLineWidth < 10 then
      vLineWidth := 10;
    vPenWidth := Round(vPenWidth * vWeight / (sgDXFLineWeights[High(sgDXFLineWeights)] / 100.0));
    if vPenWidth = 0 then Inc(vPenWidth);
    vBrush := TBrush.Create;
    try
      vBrush.Assign(ACanvas.Brush);
      ACanvas.Pen.Color := clBlack;
      ACanvas.Pen.Width := 1;
      ACanvas.Pen.Style := psSolid;
      ACanvas.Pen.Mode := pmCopy;
      ACanvas.Brush.Style := bsSolid;
      ACanvas.Brush.Color := clBlack;
      vR := Classes.Rect(vR.Left, (vR.Bottom + vR.Top - vPenWidth) div 2, vR.Left + vLineWidth,
        (vR.Bottom + vR.Top + vPenWidth) div 2 - 1);
      if vR.Bottom - vR.Top <= 0 then Inc(vR.Bottom);
      ACanvas.FillRect(vR);
      ACanvas.Brush.Color := vBrush.Color;
      {ACanvas.Pen.Width := vPenWidth;
      ACanvas.MoveTo(vR.Left + vPenWidth div 2, vR.Top + (vR.Bottom - vR.Top) div 2);
      ACanvas.LineTo(vR.Left + vLineWidth - vPenWidth div 2, vR.Top + (vR.Bottom - vR.Top) div 2);}
      ACanvas.Pen.Width := vPen.Width;
      if not (odReserved1 in State) then
        if odSelected in State then
        begin
          ACanvas.Pen.Color := clHighlightText;
          ACanvas.Brush.Color := clHighlight;
          ACanvas.Font.Color := clHighlightText;
        end
        else
        begin
          ACanvas.Pen.Color := clWindowText;
          ACanvas.Brush.Color := clWindow;
          ACanvas.Font.Color := clWindowText;
        end
      else
      begin
        ACanvas.Brush.Assign(vBrush);
        vBrush.Free;
        vBrush := nil;
        ACanvas.Pen.Assign(vPen);
        vPen.Free;
        vPen := nil;
      end;
      vLineWidth := vR.Right;
      vR := Rect;
      vR.Left := vLineWidth + 2;
      vR.Right := Rect.Right;
      sgTextRect(ACanvas.Handle, vR, S, [tfEndEllipsis, tfModifyString, tfLeft]);
      Result := PChar(S) <> AText;
    finally
      if Assigned(vBrush) then
      begin
        ACanvas.Brush.Assign(vBrush);
        vBrush.Free;
      end;
    end;
  finally
    if Assigned(vPen) then
    begin
      ACanvas.Pen.Assign(vPen);
      vPen.Free;
    end;
  end;
end;

procedure FloatCheckKeyPress(Sender: TObject; var Key: Char; IsUnsigned: Boolean);
var
  A: set of AnsiChar;
begin
  if IsUnsigned then
    A := ['1'..'9', '0', GetDecimalSeparator, #8, #13]
  else
    A := ['1'..'9', '0', GetDecimalSeparator, '-', #8, #13];
  if not CharInSet(Key, A) then
    Key := #0;
end;

procedure IntegerCheckKeyPress(Sender: TObject; var Key: Char; IsUnsigned: Boolean);
var
  A: set of AnsiChar;
begin
  A := ['0'..'9', #8, #13];
  if not IsUnsigned then
    A := A + ['-'];
  if not CharInSet(Key, A) then
    Key := #0;
end;

{ FindCompByClass

 Initialization var I needed before calling FindCompByClass. }
function FindCompByClass(AOwner: TComponent; AClass: TComponentClass;
  var I: Integer): Boolean;
var
  C: Integer;
begin
  Result := False;
  C := AOwner.ComponentCount;
  while not Result and (I < C) do
    if AOwner.Components[I].InheritsFrom(AClass) then
      Inc(Result)
    else
      Inc(I);
end;

function ShowFormatMessage(const AMessage: string; const Args: array of const;
  DlgType: TMsgDlgType; Buttons: TMsgDlgButtons): Integer;
var
  S: string;
begin
  S := Format(AMessage, Args);
  Result := MessageDlg(S, DlgType, Buttons, 0);
end;

function sgInformationMessage(const AMessage: string): Integer;
begin
  Result := MessageDlg(AMessage, mtInformation, [mbOK], 0);
end;

function sgErrorMessage(const AMessage: string): Integer;
begin
  Result := MessageDlg(AMessage, mtError, [mbOK], 0);
end;

function InpBox(ACaption, ALabel: string; var AText: string;
  const AOnCreate: TNotifyEvent = nil): Boolean;
var
  vForm: TForm;
  vEdit: TEdit;
begin
  vForm := CreateInpBox(ACaption, ALabel, AText, AOnCreate);
  try
    vEdit := TEdit(vForm.FindComponent(cnstInputBoxEditName));
    vEdit.Text := AText;
    Result := vForm.ShowModal = mrOk;
    if Result then
      AText := vEdit.Text;
  finally
    vForm.Free;
  end;
end;

function CreateInpBox(ACaption, ALabel: string; const AText: string;
  const AOnCreate: TNotifyEvent = nil): TForm;
const
  iCtrlOffs = iControlOffset;
  iBtnHeight = 23;
var
  vlblValueName: TLabel;
  vbtnOk, vbtnCancel: TButton;
  vEdit: TEdit;
begin
  Result := nil;
  try
    Result := TForm(TForm.NewInstance);
    Result.OnCreate := AOnCreate;
    Result.Create(Application);
    Result.Name := cnstInputBoxFormName;
    Result.BorderStyle := bsSizeable;
    Result.BorderIcons := [biSystemMenu];
    Result.Position := poOwnerFormCenter;
    Result.Caption := ACaption;
    vlblValueName := TLabel.Create(Result);
    vlblValueName.Name := cnstInputBoxLabelName;
    vlblValueName.Caption := ALabel;
    vEdit := TEdit.Create(Result);
    vEdit.Name := cnstInputBoxEditName;
    vEdit.OnKeyPress := nil;
    vEdit.Anchors := vEdit.Anchors + [akRight];
    vbtnCancel := TButton.Create(Result);
    vbtnCancel.Name := cnstInputBoxBtnCancelName;
    vbtnCancel.Anchors := vbtnCancel.Anchors + [akRight] - [akLeft];
    vbtnCancel.Height := iBtnHeight;
    vbtnCancel.Cancel := True;
    vbtnCancel.Caption := SMsgDlgCancel;
    vlblValueName.AutoSize := True;
    vlblValueName.Parent := Result;
    vlblValueName.Left := iCtrlOffs;
    vlblValueName.Top := 2 * iCtrlOffs;
    vEdit.Parent := Result;
    vEdit.Left := vlblValueName.BoundsRect.Right + iCtrlOffs;
    vEdit.Width := Result.ClientWidth - vEdit.Left - iCtrlOffs;
    vEdit.Top := vlblValueName.BoundsRect.Bottom - vEdit.Height;
    vEdit.Text := AText;
    vlblValueName.FocusControl := vEdit;
    vbtnCancel.Parent := Result;
    vbtnCancel.Left := Result.ClientWidth - vbtnCancel.Width - iCtrlOffs;
    vbtnCancel.Top := vlblValueName.BoundsRect.Bottom + iCtrlOffs;
    vbtnCancel.ModalResult := mrCancel;
    vbtnOk := TButton.Create(Result);
    vbtnOk.Name := cnstInputBoxBtnOKName;
    vbtnOk.Anchors := vbtnOk.Anchors + [akRight] - [akLeft];
    vbtnOk.Parent := Result;
    vbtnOk.Height := iBtnHeight;
    vbtnOk.Top := vbtnCancel.Top;
    vbtnOk.Left := vbtnCancel.Left - iCtrlOffs - vbtnOk.Width;
    vbtnOk.ModalResult := mrOk;
    vbtnOk.Default := True;
    vbtnOk.Caption := SMsgDlgOK;
    vbtnOk.TabOrder := vbtnCancel.TabOrder;
    Result.ActiveControl := vEdit;
    Result.KeyPreview := True;
    Result.ClientHeight := vbtnCancel.BoundsRect.Bottom + iCtrlOffs;
    Result.Constraints.MinHeight := Result.Height;
    Result.Constraints.MaxHeight := Result.Height;
    Result.ClientWidth := vbtnCancel.Width + vbtnOk.Width + 3 * iCtrlOffs;
    Result.Constraints.MinWidth := Result.Width;
{$IFDEF SGMULLANG}
    TsgMultiLang.Create(Result).RegisterML;
{$ENDIF}
    vEdit.SelectAll;
  except
  end;
end;

function IsSavePathValid(const APathName: string): Boolean;
var
  vWinDir: PChar;
  vPath: string;
  vXPorVista: Boolean;
begin
  Result := True;
  vWinDir := StrAlloc(MAX_PATH);
  try
    vXPorVista := ((Win32MajorVersion = 5) and (Win32MinorVersion >= 1)) or
      (Win32MajorVersion >= 6);
    if GetWindowsDirectory(vWinDir, MAX_PATH - 1) > 0 then
    begin
      vPath := AnsiUpperCase(ExtractFilePath(APathName));
      Result := not (vXPorVista and (Pos(UpperCase(vWinDir), vPath) = 1));
    end;
  finally
    StrDispose(vWinDir);
  end;
end;

function GetFileVersionStr(AFileName: string): string;
begin
  Result := GetFileResInfo(AFileName, 'FileVersion');
end;

function GetFileResInfo(const AFileName, AItem: string): string;
var
  vHandle: DWORD;
  vSize: DWORD;
  vBuffer: PChar;
  vValid: Boolean;

  function GetTranslationString: string;
  var
    P: Pointer;
    Len: Cardinal;
  begin
    VerQueryValue(vBuffer, '\VarFileInfo\Translation', P, Len);
    if P <> nil then
      Result := IntToHex(MakeLong(HiWord(Longint(P^)), LoWord(Longint(P^))), 8);
  end;

  function DoInfo(const AKey: string): string;
  var
    szName: array[0..255] of Char;
    Value: Pointer;
    Len: Cardinal;
  begin
    Result := '';
    if vValid then
    begin
      StrPCopy(szName, '\StringFileInfo\' + GetTranslationString + '\' + AKey);
      if VerQueryValue(vBuffer, szName, Value, Len) then
        Result := StrPas(PChar(Value));
    end;
  end;

begin
  vSize := GetFileVersionInfoSize(PChar(AFileName), vHandle);
  vBuffer := nil;
  if vSize > 0 then
    try
      vBuffer := AllocMem(vSize);
      vValid := GetFileVersionInfo(PChar(AFileName), vHandle, vSize, vBuffer);
    except
      vValid := False;
      raise;
    end;
  try
    if vBuffer <> nil then
      GetFileResInfo := DoInfo(AItem);
  finally
    if vBuffer <> nil then FreeMem(vBuffer, vSize);
  end;
end;

function CreateNewCrossCursor(const ASizeX, ASizeY, AApertureSize: Integer;
  const AMaskColor: TColor = 0; const AXorColor: TColor = $FFFFFF): HICON;
var
  vWidth, vHeight: Integer;
  vAndMask, vXOrMask: Graphics.TBitmap;
  vIconInfo: TIconInfo;
  R: TRect;

  procedure DoDrawCursor(const ACanvas: TCanvas; const ABack, AMask: TColor);
  begin
    ACanvas.Brush.Color := ABack;
    ACanvas.FillRect(Rect(0, 0, vWidth, vHeight));
    // Draw crosshair
    ACanvas.Pen.Color := AMask;
    ACanvas.Pen.Mode := pmXor;
    ACanvas.MoveTo(vIconInfo.xHotspot - Cardinal(ASizeX div 2), vIconInfo.yHotspot);
    ACanvas.LineTo(vIconInfo.xHotspot + Cardinal(ASizeX div 2) + 1, vIconInfo.yHotspot);
    ACanvas.MoveTo(vIconInfo.xHotspot, vIconInfo.yHotspot - Cardinal(ASizeY div 2));
    ACanvas.LineTo(vIconInfo.xHotspot, vIconInfo.yHotspot + Cardinal(ASizeY div 2) + 1);
    // Draw Aperture
    ACanvas.Brush.Style := bsClear;
    ACanvas.Rectangle(R);
  end;

begin
  vWidth := MaxI(ASizeX, AApertureSize * 2 + 1);
  vHeight := MaxI(ASizeY, AApertureSize * 2 + 1);
  vIconInfo.xHotspot := vWidth div 2;
  vIconInfo.yHotspot := vHeight div 2;
  R.Left := vIconInfo.xHotspot - Cardinal(AApertureSize);
  R.Right := vIconInfo.xHotspot + Cardinal(AApertureSize) + 1;
  R.Top := vIconInfo.yHotspot - Cardinal(AApertureSize);
  R.Bottom := vIconInfo.xHotspot + Cardinal(AApertureSize) + 1;
  vAndMask := Graphics.TBitmap.Create;
  vXOrMask := Graphics.TBitmap.Create;
  try
    vAndMask.SetSize(vWidth, vHeight);
    vXOrMask.SetSize(vWidth, vHeight);
    DoDrawCursor(vAndMask.Canvas, clWhite, AMaskColor);
    DoDrawCursor(vXOrMask.Canvas, clBlack, AXorColor);
    vIconInfo.fIcon := False;
    vIconInfo.hbmMask := vAndMask.Handle;
    vIconInfo.hbmColor := vXOrMask.Handle;
    Result := CreateIconIndirect(vIconInfo);
  finally
    vAndMask.Free;
    vXOrMask.Free;
  end;
end;

function ShellOpen(const AFileName: string; const AHandle: Cardinal): Cardinal;
var
  vHandle: Cardinal;
begin
  vHandle := AHandle;
  if vHandle = 0 then
    vHandle := Application.Handle;
  Result := ShellExecute(vHandle, 'open', PChar(AFileName), '', '',
    SW_SHOW);
end;

function IsEqualMethod(AMethod1, AMethod2: TMethod): Boolean;
begin
  Result := (AMethod1.Code = AMethod2.Code) and (AMethod1.Data = AMethod2.Data);
end;

function IsEqualEvent(AEvent1, AEvent2: TNotifyEvent): Boolean;
begin
  Result := IsEqualMethod(TMethod(AEvent1), TMethod(AEvent2));
end;

function LoadCloseIcon: Graphics.TBitmap;
const
  cnstTBEClose = 'ATBECCLOSE';
begin
  try
    Result := Graphics.TBitmap.Create;
    if (FindResource(HInstance, cnstTBEClose, RT_BITMAP) <> 0) then
    begin
      Result.Transparent := True;
      Result.LoadFromResourceName(HInstance, cnstTBEClose);
    end;
  except
    on E: Exception do
    begin
      FreeAndNil(Result);
    end;
  end;
end;

function sgIsMethodEqual(AM1, AM2: TWndMethod): Boolean;
begin
  Result := IsEqualMethod(TMethod(AM1), TMethod(AM2));
end;

//for wrap include #13#10 to Msg
function sgNotWrapMessageDialog(const Msg: string; DlgType: TMsgDlgType;
   Buttons: TMsgDlgButtons): Integer;
const
  cnstAfterLabelIdent = 16;
var
  vMsgDlg: TForm;
  I: Integer;
  vMaxWidth: Integer;
  vLabel: TLabel;
  vStrings: TStringList;
  S: string;
  vSize: tagSIZE;
begin
  vLabel := nil;
  vMaxWidth := 0;
  vStrings := TsgStringList.Create;
  try
    vStrings.Sorted := True;
    vStrings.Duplicates := dupIgnore;
    vStrings.LineBreak := #13#10;
    vStrings.Text := Msg;
    vMsgDlg := CreateMessageDialog(Msg, DlgType, Buttons);
    try
      for I := 0 to vMsgDlg.ComponentCount - 1 do
        if (vMsgDlg.Components[i] is TLabel) then
        begin
          vLabel := TLabel(vMsgDlg.Components[I]);
          vLabel.WordWrap := False;
        end;
      if Assigned(vLabel) then
      begin
        for I := 0 to vStrings.Count - 1 do
        begin
          S := vStrings[I];
          Windows.GetTextExtentPoint32(vLabel.Canvas.Handle, PChar(S), Length(S), vSize);
          if vSize.cx > vMaxWidth then
            vMaxWidth := vSize.cx;
        end;
        if vMaxWidth <> 0 then
        vMsgDlg.Width := vLabel.Left + vMaxWidth + cnstAfterLabelIdent;
      end;
      Result := vMsgDlg.ShowModal;
    finally
      vMsgDlg.Free;
    end;
  finally
    FreeAndNil(vStrings);
  end;
end;

procedure sgScanDirectory(const APath, AFileMask: string; AFiles: TStrings;
  ASearchSubfolders: Boolean = True);
var
  vPath, vFullName: string;
  vSearchRec: TSearchRec;
begin
  vPath := MakeRightPath(APath);
  if FindFirst(vPath + AFileMask, faReadOnly or faDirectory or faArchive, vSearchRec) <> 0 then
    Exit;
  try
    repeat
      if (vSearchRec.Name = '.') or (vSearchRec.Name = '..') then
        Continue;
      vFullName := vPath + vSearchRec.Name;
      if vSearchRec.Attr and faDirectory = 0 then
        AFiles.Add(vFullName)
      else
        if ASearchSubfolders then
          sgScanDirectory(vFullName, AFileMask, AFiles, ASearchSubfolders);
    until FindNext(vSearchRec) <> 0;
  finally
    FindClose(vSearchRec.FindHandle);
  end;
end;

function WebFileExt(const AFilelName: string): string;
var
  I: Integer;
  S: string;
begin
  S := LowerCase(AFilelName);
  I := Length(S);
  Result := '';
  if I > 0 then
    repeat
      Result := S[I] + Result;
      Dec(I);
    until (I <= 0) or (Result[1] = '.');
end;

procedure ResetSHXFontsHash;
begin
  SHX.RefreshHashLists;
end;

function CreateMetafileFromCadImage(const AImg: TsgCADImage;
  const AIsEnhanced: Boolean; const APicWidth, APicHeight, APicInch: Integer;
  const ADrawProc: TsgObjProcDraw; const AUseLogicMetrix: Boolean): TMetafile;
var
  vWidth, vHeight: Integer;
begin
  Result := TMetafile.Create;
  try
    AImg.Stretch := False;
    AImg.BackgroundColor := clWhite;
    AImg.DefaultColor := clBlack;
    AImg.BorderSize := 0;
    AImg.BorderType := btGlobal;
    AImg.Transparent := True;
    Result.Enhanced := AIsEnhanced;
    if APicInch = 0 then
      Result.Inch := 96
    else
      Result.Inch := APicInch;
    if AUseLogicMetrix then
    begin
      Result.Width := APicWidth;
      Result.Height := APicHeight;
    end
    else
    begin
      SetMaxMetafileExtents(APicWidth, APicHeight, vWidth, vHeight,
        GetDefaultMetafileSize(AIsEnhanced));
      Result.Width := vWidth;
      Result.Height := vHeight;
    end;
    TsgCADImageAccess(AImg).DrawOnMetafile(Result, AUseLogicMetrix, ADrawProc);
  except
    FreeAndNil(Result);
  end;
end;

function GetMetafileSizeFromCadImage(const AImg: TsgCADImage): TPoint;
var
  vSize: TF2DPoint;
  vMaxDim: Double;
begin
  vSize := AImg.AbsSize;
  vMaxDim := Max(vSize.X, vSize.Y);
  if vMaxDim < iMetafileSizeMin  then
    SetMaxSize(iMetafileSizeMin, vSize.X, vSize.Y)
  else
    if vMaxDim > iMetafileSizeMax then
      SetMaxSize(iMetafileSizeMax, vSize.X, vSize.Y);
  Result.X := Ceil(vSize.X);
  Result.Y := Ceil(vSize.Y);
end;


end.
